/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QWidget>
#include <interfaces/ihavetabs.h>
#include <interfaces/core/icoreproxy.h>

class QMenu;
class QTermWidget;

namespace LeechCraft
{
namespace Util
{
	class ShortcutManager;
}

namespace Eleeminator
{
	class TermTab : public QWidget
				  , public ITabWidget
	{
		Q_OBJECT
		Q_INTERFACES (ITabWidget)

		const ICoreProxy_ptr CoreProxy_;
		const TabClassInfo TC_;
		QObject * const ParentPlugin_;

		QToolBar * const Toolbar_;

		QTermWidget * const Term_;

		QString CurrentColorScheme_;

		bool IsTabCurrent_ = false;
	public:
		TermTab (const ICoreProxy_ptr&, Util::ShortcutManager*, const TabClassInfo&, QObject*);

		TabClassInfo GetTabClassInfo () const;
		QObject* ParentMultiTabs ();
		QToolBar* GetToolBar () const;
		void Remove ();
		void TabMadeCurrent ();
		void TabLostCurrent ();
	private:
		void SetupToolbar (Util::ShortcutManager*);
		void SetupColorsButton ();
		void SetupFontsButton ();

		void SetupShortcuts (Util::ShortcutManager*);

		void AddUrlActions (QMenu&, const QPoint&);
		void AddLocalFileActions (QMenu&, const QString&);
	private slots:
		void setHistorySettings ();

		void handleTermContextMenu (const QPoint&);

		void openUrl ();
		void copyUrl ();

		void setColorScheme (QAction*);
		void previewColorScheme (QAction*);
		void stopColorSchemePreview ();

		void selectFont ();

		void updateTitle ();

		void handleUrlActivated (const QUrl&);
		void handleBell (const QString&);

		void handleFinished ();
	signals:
		void changeTabName (QWidget*, const QString&);
		void remove (QWidget*);
	};
}
}
