/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2011  Minh Ngo
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#ifndef PLUGINS_OTZERKALU_OTZERKALUDOWNLOADER_H
#define PLUGINS_OTZERKALU_OTZERKALUDOWNLOADER_H
#include <QObject>
#include <QUrl>
#include <QWebFrame>
#include <QWebElementCollection>
#include <interfaces/structures.h>
#include <interfaces/ientityhandler.h>

namespace LeechCraft
{
namespace Otzerkalu
{
	struct DownloadParams
	{
		QUrl DownloadUrl_;
		QString DestDir_;
		int RecLevel_;
		bool FromOtherSite_;

		DownloadParams ();
		DownloadParams (const QUrl& downloadUrl, const QString& destDir,
				int recLevel, bool fromOtherSite);
	};

	struct FileData
	{
		QUrl Url_;
		QString Filename_;
		int RecLevel_;

		FileData ();
		FileData (const QUrl& url, const QString& filename, int recLevel);
	};

	class OtzerkaluDownloader : public QObject
	{
		Q_OBJECT
		const DownloadParams Param_;
		QMap<int, FileData> FileMap_;
		QStringList DownloadedFiles_;
		int UrlCount_, ID_;
	public:
		OtzerkaluDownloader (const DownloadParams& param, int id, QObject *parent = 0);
		QString GetLastDownloaded () const;
		int FilesCount () const;
		void Begin ();
	private:
		QString Download (const QUrl&, int);
		QList<QUrl> CSSParser (const QString&) const;
		QString CSSUrlReplace (const QString&, const FileData&);
		bool HTMLReplace (QWebElementCollection::iterator element, const FileData& data);
		bool WriteData (const QString& filename, const QString& data);
		void HandleProvider (QObject *provider, int id, const QUrl& url,
				const QString& filename, int recLevel);
	private slots:
		void handleJobFinished (int id);
	signals:
		void delegateEntity (const LeechCraft::Entity&, int*, QObject**);
		void gotEntity (const LeechCraft::Entity&);
		void fileDownloaded (int id, int count);
		void mirroringFinished (int id);
	};
};
};

#endif // PLUGINS_OTZERKALU_OTZERKALUDOWNLOADER_H
