/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <memory>
#include <QObject>
#include <QUrl>
#include <QHash>
#include <QSet>
#include <interfaces/core/icoreproxy.h>
#include <interfaces/iquarkcomponentprovider.h>

class QSettings;
class QMainWindow;
class QDir;
class QStandardItemModel;
class QToolBar;

namespace LeechCraft
{
class QuarkComponent;

namespace Util
{
	class ShortcutManager;
}

namespace SB2
{
	class ViewSettingsManager;
	class SBView;
	class QuarkManager;
	class ViewGeometryManager;

	typedef std::shared_ptr<QuarkManager> QuarkManager_ptr;

	class ViewManager : public QObject
	{
		Q_OBJECT

		ICoreProxy_ptr Proxy_;
		QStandardItemModel *ViewItemsModel_;
		SBView *View_;
		QToolBar *Toolbar_;

		QMainWindow *Window_;
		const bool IsDesktopMode_;
		const int OnloadWindowIndex_;

		QHash<QUrl, QuarkManager_ptr> Quark2Manager_;
		QSet<QString> RemovedIDs_;
		QStringList PreviousQuarkOrder_;

		QuarkComponents_t InternalComponents_;

		ViewSettingsManager *SettingsManager_;

		ViewGeometryManager *GeomManager_;
	public:
		ViewManager (ICoreProxy_ptr, Util::ShortcutManager*, QMainWindow*, QObject* = 0);

		SBView* GetView () const;
		QToolBar* GetToolbar () const;
		QMainWindow* GetManagedWindow () const;
		QRect GetFreeCoords () const;
		ViewSettingsManager* GetViewSettingsManager () const;

		bool IsDesktopMode () const;

		void SecondInit ();
		void RegisterInternalComponent (QuarkComponent_ptr);

		void ShowSettings (const QUrl&);

		void RemoveQuark (const QUrl& loadedUrl);
		void RemoveQuark (const QString& id);
		void UnhideQuark (QuarkComponent_ptr, QuarkManager_ptr);
		void MoveQuark (int from, int to);

		void MovePanel (Qt::ToolBarArea);

		QuarkComponents_t FindAllQuarks () const;
		QList<QUrl> GetAddedQuarks () const;
		QuarkManager_ptr GetAddedQuarkManager (const QUrl&) const;

		int GetWindowIndex () const;

		std::shared_ptr<QSettings> GetSettings () const;
	private:
		void AddComponent (QuarkComponent_ptr, bool forceAdd);
		void AddComponent (QuarkComponent_ptr, QuarkManager_ptr, bool forceAdd);
		QuarkComponents_t ScanRootDir (const QDir&) const;

		void AddToRemoved (const QString&);
		void RemoveFromRemoved (const QString&);

		void SaveRemovedList () const;
		void LoadRemovedList ();

		void SaveQuarkOrder ();
		void LoadQuarkOrder ();
	private slots:
		void handleQuarksAdded (const QList<QUrl>&);
		void handleQuarksRemoved (const QList<QUrl>&);
	};
}
}
