/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "attdescr.h"
#include <QtDebug>
#include "vmimeconversions.h"

namespace LeechCraft
{
namespace Snails
{
	AttDescr::AttDescr ()
	: Size_ (0)
	{
	}

	AttDescr::AttDescr (vmime::shared_ptr<const vmime::attachment> att)
	: Name_ (StringizeCT (att->getName ()))
	, Descr_ (StringizeCT (att->getDescription ()))
	, Size_ (att->getData ()->getLength ())
	, Type_ (att->getType ().getType ().c_str ())
	, SubType_ (att->getType ().getSubType ().c_str ())
	{
	}

	AttDescr::AttDescr (const QString& name, const QString& descr,
			const QByteArray& type, const QByteArray& subtype,
			qlonglong size)
	: Name_ (name)
	, Descr_ (descr)
	, Size_ (size)
	, Type_ (type)
	, SubType_ (subtype)
	{
	}

	QString AttDescr::GetName () const
	{
		return Name_;
	}

	QString AttDescr::GetDescr () const
	{
		return Descr_;
	}

	qlonglong AttDescr::GetSize () const
	{
		return Size_;
	}

	QByteArray AttDescr::GetType () const
	{
		return Type_;
	}

	QByteArray AttDescr::GetSubType () const
	{
		return SubType_;
	}

	QByteArray AttDescr::Serialize () const
	{
		QByteArray res;

		QDataStream str (&res, QIODevice::WriteOnly);
		str << static_cast<quint8> (1)
			<< Name_
			<< Descr_
			<< Size_
			<< Type_
			<< SubType_;

		return res;
	}

	void AttDescr::Deserialize (const QByteArray& serialized)
	{
		QDataStream str (serialized);

		quint8 version = 0;
		str >> version;
		if (version != 1)
			throw std::runtime_error (qPrintable ("Failed to deserialize AttDescr: unknown version " + QString::number (version)));

		str >> Name_
			>> Descr_
			>> Size_
			>> Type_
			>> SubType_;
	}

	void AttDescr::Dump () const
	{
		qDebug () << Q_FUNC_INFO
				<< Name_
				<< Type_ + '/' + SubType_
				<< Size_
				<< Descr_;
	}

	QDataStream& operator<< (QDataStream& str, const AttDescr& descr)
	{
		str << descr.Serialize ();
		return str;
	}

	QDataStream& operator>> (QDataStream& str, AttDescr& descr)
	{
		QByteArray data;
		str >> data;
		descr.Deserialize (data);
		return str;
	}
}
}
