/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "tpi.h"
#include <QIcon>
#include <QAbstractItemModel>
#include <util/sys/paths.h>
#include <util/gui/util.h>
#include "infomodelmanager.h"

#if QT_VERSION < 0x050000
#include "tooltipview.h"
#endif

namespace LeechCraft
{
namespace TPI
{
	void Plugin::Init (ICoreProxy_ptr proxy)
	{
		Proxy_ = proxy;

		ModelMgr_ = new InfoModelManager (proxy);

		QuarkComponent_ptr comp (new QuarkComponent ("tpi", "TPIQuark.qml"));
		comp->DynamicProps_ << QPair<QString, QObject*> ("TPI_infoModel", ModelMgr_->GetModel ());
#if QT_VERSION < 0x050000
		comp->DynamicProps_ << QPair<QString, QObject*> ("TPI_proxy", this);
#endif
		Components_ << comp;
	}

	void Plugin::SecondInit ()
	{
		ModelMgr_->SecondInit ();
	}

	QByteArray Plugin::GetUniqueID () const
	{
		return "org.LeechCraft.TPI";
	}

	void Plugin::Release ()
	{
	}

	QString Plugin::GetName () const
	{
		return "TPI";
	}

	QString Plugin::GetInfo () const
	{
		return tr ("Task Progress Indicator quark plugin.");
	}

	QIcon Plugin::GetIcon () const
	{
		return QIcon ();
	}

	QuarkComponents_t Plugin::GetComponents () const
	{
		return Components_;
	}

#if QT_VERSION < 0x050000
	void Plugin::hovered (int x, int y, const QRect& geometry)
	{
		if (!TooltipView_)
			TooltipView_ = new TooltipView (ModelMgr_->GetModel (), Proxy_->GetColorThemeManager ());

		TooltipView_->move (Util::FitRect ({ x, y },
				TooltipView_->size (), geometry, Util::NoOverlap));
		TooltipView_->show ();
		TooltipView_->Hovered ();
	}

	void Plugin::hoverLeft ()
	{
		if (TooltipView_)
			TooltipView_->Unhovered ();
	}
#endif
}
}

LC_EXPORT_PLUGIN (leechcraft_tpi, LeechCraft::TPI::Plugin);
